@@ -455,11 +455,19 @@ setMethod("get.most.probable.values",
 
 
 # redefition of print() for BN objects
+#' print a \code{\link{BN}}, \code{\link{BNDataset}} or \code{\link{InferenceEngine}} to \code{stdout}.
+#' 
+#' @name print
+#' 
+#' @param x a \code{\link{BN}}, \code{\link{BNDataset}} or \code{\link{InferenceEngine}}.
+# ' @param ... potential other arguments.
+#'
 #' @rdname print
-#' @aliases print,BN
-setMethod("print",
-          "BN",
-          function(x, ...)
+#' @aliases print,BN print.BN,BN
+#' @export
+#setMethod("print.BN",
+#          "BN",
+print.BN <- function(x, ...)
           {
             str <- "\nBayesian Network: "
             str <- paste(str, name(x), sep = '')
@@ -510,18 +518,23 @@ setMethod("print",
             
             cat("\n")
             
-          })
+          }#)
 
 
 #' plot a \code{\link{BN}} as a picture.
 #'
 #' @param x a \code{\link{BN}} object.
-#' @param ... potential further arguments for methods.
+#' @param method either \code{default} of \code{qgraph}. The \code{default} method requires
+#'        the \code{Rgraphviz} package, while \code{qgraph} requires the \code{qgraph} package
+#'        and allows for a greater customization.
 #' @param use.node.names \code{TRUE} if node names have to be printed. If \code{FALSE}, numbers are used instead.
+#' @param node.size.lab font size for the node labels in the default mode.
 #' @param node.col list of (\code{R}) colors for the nodes.
 #' @param plot.wpdag if \code{TRUE} plot the network according to the WPDAG computed using bootstrap instead of the DAG.
 #' @param frac minimum fraction [0,1] of presence of an edge to be plotted (used in case of \code{plot.wpdag=TRUE}).
 #' @param max.weight maximum possible weight of an edge (used in case of \code{plot.wpdag=TRUE}).
+#' @param ... potential further arguments when using \code{method="qgraph"}. Please refer to the
+#'        \code{qgraph} documentation for the parameters available for the \code{qgraph()} method.
 #' 
 #' @importFrom graphics plot
 #' @importFrom grDevices colors dev.off postscript
@@ -531,16 +544,25 @@ setMethod("print",
 #' @rdname plot
 #' @export
 plot.BN <- 
-  function( x, ..., use.node.names = TRUE, frac = 0.2, 
-                    max.weight = max(dag(x)), node.col = rep('white',num.nodes(x)),
-                    plot.wpdag = FALSE)
+  function( x, method = "default", use.node.names = TRUE, frac = 0.2, max.weight = max(dag(x)),
+                    node.size.lab=14, node.col = rep('white',num.nodes(x)),
+                    plot.wpdag = FALSE, ...)
           {
             
-            # check for Rgraphviz
-            if (!requireNamespace("Rgraphviz", quietly=T))
-              stop("this function requires the Rgraphviz package.")
+            # check for required packages
             if (!requireNamespace("graph", quietly=T))
               stop("this function requires the graph package.")
+            method <- tolower(method)
+            if (method == "default") {
+                if (!requireNamespace("Rgraphviz", quietly=T))
+                    stop("this function requires the Rgraphviz package.")
+            } else if (method == "qgraph") {
+                if (!requireNamespace("qgraph", quietly=T))
+                    stop("this function requires the qgraph package when using 'method = \"qgraph\"'.") 
+            } else {
+                stop("plotting method not available in bnstruct. Please use one between 'default' and 'qgraph'.")
+            }
+
             
             # adjacency matrix
             if (plot.wpdag || (!is.element(1,dag(x)) && length(which(wpdag(x) != 0)) > 0))
@@ -558,13 +580,23 @@ plot.BN <-
             
             num.nodes <- num.nodes(x)
             variables <- variables(x)
-            
-            mat.th <- mat
-            if (is.element(1,dag(x)) || length(which(wpdag(x) != 0)) > 0)
-            {
-              mat.th[mat <  frac*max.weight] <- 0
-              mat.th[mat >= frac*max.weight] <- 1
+
+            if (method == "default") {
+                bngzplot(x, mat, num.nodes, variables, use.node.names,
+                            frac, max.weight, node.size.lab, node.col)
+            } else {
+                plist <- list(...)
+                bnqgplot(mat, num.nodes, variables, use.node.names,
+                            frac, max.weight, node.col, plist)
             }
+}            
+
+bngzplot <- function(x, mat, num.nodes, variables, use.node.names, frac, max.weight,
+                      node.size.lab, node.col) {
+
+            mat.th <- mat
+            mat.th[mat <  frac*max.weight] <- 0
+            mat.th[mat >= frac*max.weight] <- 1
             
             # node names
             if (use.node.names && length(variables) > 0)
@@ -574,10 +606,16 @@ plot.BN <-
             # build graph
             rownames(mat.th) <- node.names
             colnames(mat.th) <- node.names
+
+            # node colors
+            node.fill <- as.list(node.col)
+            names(node.fill) <- node.names
+
             g <- new("graphAM", adjMat=mat.th, edgemode="directed")
+
             en <- Rgraphviz::edgeNames(g,recipEdges="distinct")
             g <- Rgraphviz::layoutGraph(g)
-            
+
             # set edge darkness proportional to confidence
             conf <- mat.th*pmax(mat,t(mat)) # both values to the maximum for edges with 2 directions
             col <- colors()[253-100*(t(conf)[t(conf) >= frac*max.weight]/max.weight)]
@@ -589,28 +627,63 @@ plot.BN <-
             dirs <- graph::edgeRenderInfo(g)$direction
             ahs[dirs=="both"] <- ats[dirs=="both"] <- "none"
             graph::edgeRenderInfo(g) <- list(col=col,lwd=2,arrowhead=ahs,arrowtail=ats)
-            
+            graph::nodeRenderInfo(g) <- list(fill=node.fill, fontsize=node.size.lab)
+
+            Rgraphviz::renderGraph(g)
+}
+
+bnqgplot <- function(mat, num.nodes, variables, use.node.names,
+                    frac, max.weight, node.col, ...) {
+
+            # parse ... parameters
+            plist <- unlist(list(...), recursive=F)
+            parnames <- names(plist)
+
+            if (use.node.names && length(variables) > 0)
+              node.names <- variables
+            else
+              node.names <- as.character(1:num.nodes)
+
+            # build graph
+            rownames(mat) <- node.names
+            colnames(mat) <- node.names
+
             # node colors
             node.fill <- as.list(node.col)
             names(node.fill) <- node.names
-            graph::nodeRenderInfo(g) <- list(fill=node.fill)
-            
-            Rgraphviz::renderGraph(g)
-          }
+            plist[["input"]] <- mat
+
+            if (!("color" %in% parnames)) {
+                plist[["color"]] <- node.col
+            }
+            if (!("minumum" %in% parnames)) {
+                plist[["minimum"]] <- frac * max.weight
+            }
+            if (!("directed" %in% parnames)) {
+                plist[["directed"]] <- TRUE
+            }
+
+            if (!("labels" %in% parnames)) {
+                plist[["labels"]] <- node.names
+            }
+
+            do.call(qgraph::qgraph, plist)
+}
+
 
 # save BN as eps file
 #' @rdname save.to.eps
 #' @aliases save.to.eps,BN,character
 setMethod("save.to.eps",
           c("BN", "character"),
-          function(x, filename)
+          function(x, filename, ...)
           {
             # problem: I wanted to set filename=NULL in the declaration, but I cannot manage to
             # make it work in case of missing filename...
             
             # problem 2: cannot make dag.to.cpdag work...
             postscript(filename)
-            plot(x)
+            plot(x, ...)
             dev.off()
           })
 
@@ -731,6 +804,77 @@ dag.to.cpdag <- function(dag.adj.matrix, layering = NULL)
 }
 
 
+#' counts the edges in a WPDAG with their directionality
+#'
+#' Given a \code{BN} with a \code{WPDAG}, it counts the edges, with
+#' their directionality.
+#'
+#' @param x the \code{BN}
+#' @param use.node.names use node names rather than number (\code{TRUE} by default).
+#'
+#' @return a matrix containing the node pairs with the count of the edges
+#'         between them in the \code{WPDAG}.
+#'
+#' @name edge.dir.wpdag
+#' @rdname edge.dir.wpdag
+#'
+#' @export
+edge.dir.wpdag <- function (x, use.node.names = TRUE)
+{
+  if (!is.element(1, dag(x)) && length(which(wpdag(x) != 0)) > 0)
+    mat <- wpdag(x)
+  else
+    mat <- dag(x)
+
+  num.nodes <- num.nodes(x)
+  variables <- variables(x)
+
+    if (use.node.names && length(variables) > 0)
+    node.names <- variables
+  else node.names <- as.character(1:num.nodes)
+
+  rownames(mat) <- node.names
+  colnames(mat) <- node.names
+
+  # Indexes of element inside WPDAG lower triangular matrix
+  ltel <- which(lower.tri(mat, diag = FALSE), arr.ind=T)
+  # Respective elements inside WPDAG upper triangular matrix
+  utel <- cbind(ltel[,2], ltel[,1])
+
+  # Find pairs of nodes [i,j] with an edge directed from i to j
+  nodes.dir.l <- ltel[which(mat[ltel]>mat[utel]), ]
+  nodes.dir.u <- utel[which(mat[utel]>mat[ltel]), ]
+
+  ### Find number of occurrences of directed edges ###
+  # Count edges occurence for each pair of nodes
+  edge.occurr.l <- mat[nodes.dir.l]
+  edge.occurr.u <- mat[nodes.dir.u]
+  edge.occurr <- c(edge.occurr.l, edge.occurr.u)
+  # Once pairs of nodes with directed edge have been found, count the occurences of that edge in the opposite direction
+  nodes.rev.l <- cbind(nodes.dir.l[,2], nodes.dir.l[,1])
+  nodes.rev.u <- cbind(nodes.dir.u[,2], nodes.dir.u[,1])
+  edge.rev.occurr.l <- mat[nodes.rev.l]
+  edge.rev.occurr.u <- mat[nodes.rev.u]
+  edge.rev.occurr <- c(edge.rev.occurr.l, edge.rev.occurr.u)
+
+  # Name of nodes with directed edges
+  nodes.start.l <- rownames(mat)[nodes.dir.l[,1]]
+  nodes.stop.l <- rownames(mat)[nodes.dir.l[,2]]
+
+  nodes.start.u <- rownames(mat)[nodes.dir.u[,1]]
+  nodes.stop.u <- rownames(mat)[nodes.dir.u[,2]]
+  # Combine together pairs of nodes (edges from nodes.start to nodes.stop)
+  nodes.start <- c(nodes.start.l, nodes.start.u)
+  nodes.stop <- c(nodes.stop.l, nodes.stop.u)
+  nodes.dir <- cbind(nodes.start, nodes.stop)
+  # Combine pairs of nodes (names) with info about edges occurrence 
+  edge.directed.wpdag <- cbind(nodes.dir, edge.occurr, edge.rev.occurr)
+
+  return(edge.directed.wpdag)
+}
+
+
+
 label.edges <- function(dgraph, layering = NULL)
 {
   # LABEL-EDGES produce a N*N matrix which values are
